@@ -32,15 +32,13 @@
 #include <QSettings>
 #include <QByteArray>
 #include <QMessageBox>
+#include <QThread>
 
-#if QT_VERSION >= 0x050900
 #include <QOperatingSystemVersion>
-#endif
 
 #include <time.h>
 
 #include <KisApplication.h>
-#include <KisLoggingManager.h>
 #include <KoConfig.h>
 #include <KoResourcePaths.h>
 #include <kis_config.h>
@@ -55,15 +53,24 @@
 #include "KisApplicationArguments.h"
 #include <opengl/kis_opengl.h>
 #include "input/KisQtWidgetsTweaker.h"
+#include <KisUsageLogger.h>
+#include <kis_image_config.h>
 
 #if defined Q_OS_WIN
+#include "config_use_qt_tablet_windows.h"
 #include <windows.h>
+#ifndef USE_QT_TABLET_WINDOWS
 #include <kis_tablet_support_win.h>
 #include <kis_tablet_support_win8.h>
+#else
+#include <dialogs/KisDlgCustomTabletResolution.h>
+#endif
+#include "config-high-dpi-scale-factor-rounding-policy.h"
+#include "config-set-has-border-in-full-screen-default.h"
+#ifdef HAVE_SET_HAS_BORDER_IN_FULL_SCREEN_DEFAULT
+#include <QtPlatformHeaders/QWindowsWindowFunctions>
+#endif
 #include <QLibrary>
-
-#elif defined HAVE_X11
-#include <kis_xi2_event_filter.h>
 #endif
 
 #if defined HAVE_KCRASH
@@ -93,7 +100,12 @@ void tryInitDrMingw()
     QString logFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation).replace(L'/', L'\\') + QStringLiteral("\\kritacrash.log");
     myExcHndlSetLogFileNameA(logFile.toLocal8Bit());
 }
+} // namespace
+#endif
 
+#ifdef Q_OS_WIN
+namespace
+{
 typedef enum ORIENTATION_PREFERENCE {
     ORIENTATION_PREFERENCE_NONE = 0x0,
     ORIENTATION_PREFERENCE_LANDSCAPE = 0x1,
@@ -122,6 +134,7 @@ void resetRotation()
 }
 } // namespace
 #endif
+
 extern "C" int main(int argc, char **argv)
 {
 
@@ -133,7 +146,8 @@ extern "C" int main(int argc, char **argv)
     qputenv("QT_QPA_PLATFORM", "xcb");
 #endif
 
-    KisLoggingManager::initialize();
+    // Workaround a bug in QNetworkManager
+    qputenv("QT_BEARER_POLL_TIMEOUT", QByteArray::number(-1));
 
     // A per-user unique string, without /, because QLocalServer cannot use names with a / in it
     QString key = "Krita4" + QStandardPaths::writableLocation(QStandardPaths::HomeLocation).replace("/", "_");
@@ -144,26 +158,48 @@ extern "C" int main(int argc, char **argv)
     QCoreApplication::setAttribute(Qt::AA_DontCreateNativeWidgetSiblings, true);
     QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 
-#if QT_VERSION >= 0x050900
     QCoreApplication::setAttribute(Qt::AA_DisableShaderDiskCache, true);
+
+#ifdef HAVE_HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY
+    // This rounding policy depends on a series of patches to Qt related to
+    // https://bugreports.qt.io/browse/QTBUG-53022. These patches are applied
+    // in ext_qt for WIndows (patches 0031-0036).
+    //
+    // The rounding policy can be set externally by setting the environment
+    // variable `QT_SCALE_FACTOR_ROUNDING_POLICY` to one of the following:
+    //   Round:            Round up for .5 and above.
+    //   Ceil:             Always round up.
+    //   Floor:            Always round down.
+    //   RoundPreferFloor: Round up for .75 and above.
+    //   PassThrough:      Don't round.
+    //
+    // The default is set to RoundPreferFloor for better behaviour than before,
+    // but can be overridden by the above environment variable.
+    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor);
 #endif
 
     const QString configPath = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation);
+    QSettings kritarc(configPath + QStringLiteral("/kritadisplayrc"), QSettings::IniFormat);
 
     bool singleApplication = true;
     bool enableOpenGLDebug = false;
     bool openGLDebugSynchronous = false;
+    bool logUsage = true;
     {
-        QSettings kritarc(configPath + QStringLiteral("/kritadisplayrc"), QSettings::IniFormat);
+
         singleApplication = kritarc.value("EnableSingleApplication", true).toBool();
-#if QT_VERSION >= 0x050600
-        if (kritarc.value("EnableHiDPI", false).toBool()) {
+        if (kritarc.value("EnableHiDPI", true).toBool()) {
             QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
         }
         if (!qgetenv("KRITA_HIDPI").isEmpty()) {
             QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
         }
+#ifdef HAVE_HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY
+        if (kritarc.value("EnableHiDPIFractionalScaling", true).toBool()) {
+            QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
+        }
 #endif
+
         if (!qgetenv("KRITA_OPENGL_DEBUG").isEmpty()) {
             enableOpenGLDebug = true;
         } else {
@@ -173,24 +209,34 @@ extern "C" int main(int argc, char **argv)
             openGLDebugSynchronous = true;
         }
 
-        KisOpenGL::setDefaultFormat(enableOpenGLDebug, openGLDebugSynchronous);
+        KisConfig::RootSurfaceFormat rootSurfaceFormat = KisConfig::rootSurfaceFormat(&kritarc);
+        KisOpenGL::OpenGLRenderer preferredRenderer = KisOpenGL::RendererAuto;
+
+        logUsage = kritarc.value("LogUsage", true).toBool();
 
 #ifdef Q_OS_WIN
-        QString preferredOpenGLRenderer = kritarc.value("OpenGLRenderer", "auto").toString();
+        const QString preferredRendererString = kritarc.value("OpenGLRenderer", "angle").toString();
+#else
+        const QString preferredRendererString = kritarc.value("OpenGLRenderer", "auto").toString();
+#endif
+        preferredRenderer = KisOpenGL::convertConfigToOpenGLRenderer(preferredRendererString);
 
-        // Force ANGLE to use Direct3D11. D3D9 doesn't support OpenGL ES 3 and WARP
-        //  might get weird crashes atm.
-        qputenv("QT_ANGLE_PLATFORM", "d3d11");
+        const KisOpenGL::RendererConfig config =
+            KisOpenGL::selectSurfaceConfig(preferredRenderer, rootSurfaceFormat, enableOpenGLDebug);
 
-        // Probe QPA auto OpenGL detection
-        char *fakeArgv[2] = { argv[0], nullptr }; // Prevents QCoreApplication from modifying the real argc/argv
-        KisOpenGL::probeWindowsQpaOpenGL(1, fakeArgv, preferredOpenGLRenderer);
+        KisOpenGL::setDefaultSurfaceConfig(config);
+        KisOpenGL::setDebugSynchronous(openGLDebugSynchronous);
 
+#ifdef Q_OS_WIN
         // HACK: https://bugs.kde.org/show_bug.cgi?id=390651
         resetRotation();
 #endif
     }
 
+    if (logUsage) {
+        KisUsageLogger::initialize();
+    }
+
 
     QString root;
     QString language;
@@ -224,13 +270,20 @@ extern "C" int main(int argc, char **argv)
     // selection dialog.
 
     dbgKrita << "Override language:" << language;
-
+    bool rightToLeft = false;
     if (!language.isEmpty()) {
         KLocalizedString::setLanguages(language.split(":"));
         // And override Qt's locale, too
         qputenv("LANG", language.split(":").first().toLocal8Bit());
         QLocale locale(language.split(":").first());
         QLocale::setDefault(locale);
+
+        const QStringList rtlLanguages = QStringList()
+                << "ar" << "dv" << "he" << "ha" << "ku" << "fa" << "ps" << "ur" << "yi";
+
+        if (rtlLanguages.contains(language.split(':').first())) {
+            rightToLeft = true;
+        }
     }
     else {
         dbgKrita << "Qt UI languages:" << QLocale::system().uiLanguages() << qgetenv("LANG");
@@ -277,8 +330,40 @@ extern "C" int main(int argc, char **argv)
         }
     }
 
+#if defined Q_OS_WIN && defined USE_QT_TABLET_WINDOWS && defined QT_HAS_WINTAB_SWITCH
+    const bool forceWinTab = !KisConfig::useWin8PointerInputNoApp(&kritarc);
+    QCoreApplication::setAttribute(Qt::AA_MSWindowsUseWinTabAPI, forceWinTab);
+
+    if (qEnvironmentVariableIsEmpty("QT_WINTAB_DESKTOP_RECT") &&
+        qEnvironmentVariableIsEmpty("QT_IGNORE_WINTAB_MAPPING")) {
+
+        QRect customTabletRect;
+        KisDlgCustomTabletResolution::Mode tabletMode =
+            KisDlgCustomTabletResolution::getTabletMode(&customTabletRect);
+        KisDlgCustomTabletResolution::applyConfiguration(tabletMode, customTabletRect);
+    }
+#endif
+
     // first create the application so we can create a pixmap
     KisApplication app(key, argc, argv);
+    KisUsageLogger::writeHeader();
+    KisOpenGL::initialize();
+
+#ifdef HAVE_SET_HAS_BORDER_IN_FULL_SCREEN_DEFAULT
+    if (QCoreApplication::testAttribute(Qt::AA_UseDesktopOpenGL)) {
+        QWindowsWindowFunctions::setHasBorderInFullScreenDefault(true);
+    }
+#endif
+
+
+    if (!language.isEmpty()) {
+        if (rightToLeft) {
+            app.setLayoutDirection(Qt::RightToLeft);
+        }
+        else {
+            app.setLayoutDirection(Qt::LeftToRight);
+        }
+    }
     KLocalizedString::setApplicationDomain("krita");
 
     dbgKrita << "Available translations" << KLocalizedString::availableApplicationTranslations();
@@ -289,10 +374,10 @@ extern "C" int main(int argc, char **argv)
     QDir appdir(KoResourcePaths::getApplicationRoot());
     QString path = qgetenv("PATH");
     qputenv("PATH", QFile::encodeName(appdir.absolutePath() + "/bin" + ";"
-                                    + appdir.absolutePath() + "/lib" + ";"
-                                    + appdir.absolutePath() + "/Frameworks" + ";"
-                                    + appdir.absolutePath() + ";"
-                                    + path));
+                                      + appdir.absolutePath() + "/lib" + ";"
+                                      + appdir.absolutePath() + "/Frameworks" + ";"
+                                      + appdir.absolutePath() + ";"
+                                      + path));
 
     dbgKrita << "PATH" << qgetenv("PATH");
 #endif
@@ -318,7 +403,7 @@ extern "C" int main(int argc, char **argv)
     if (singleApplication && app.isRunning()) {
         // only pass arguments to main instance if they are not for batch processing
         // any batch processing would be done in this separate instance
-        const bool batchRun = args.exportAs();
+        const bool batchRun = args.exportAs() || args.exportSequence();
 
         if (!batchRun) {
             QByteArray ba = args.serialize();
@@ -332,13 +417,11 @@ extern "C" int main(int argc, char **argv)
         // Icons in menus are ugly and distracting
         app.setAttribute(Qt::AA_DontShowIconsInMenus);
     }
-
-#if defined HAVE_X11
-    app.installNativeEventFilter(KisXi2EventFilter::instance());
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+    app.setAttribute(Qt::AA_DisableWindowContextHelpButton);
 #endif
     app.installEventFilter(KisQtWidgetsTweaker::instance());
 
-
     if (!args.noSplash()) {
         // then create the pixmap from an xpm: we cannot get the
         // location of our datadir before we've started our components,
@@ -357,9 +440,8 @@ extern "C" int main(int argc, char **argv)
     }
 
 #if defined Q_OS_WIN
-    KisConfig cfg;
+    KisConfig cfg(false);
     bool supportedWindowsVersion = true;
-#if QT_VERSION >= 0x050900
     QOperatingSystemVersion osVersion = QOperatingSystemVersion::current();
     if (osVersion.type() == QOperatingSystemVersion::Windows) {
         if (osVersion.majorVersion() >= QOperatingSystemVersion::Windows7.majorVersion()) {
@@ -378,8 +460,7 @@ extern "C" int main(int argc, char **argv)
             }
         }
     }
-#endif
-
+#ifndef USE_QT_TABLET_WINDOWS
     {
         if (cfg.useWin8PointerInput() && !KisTabletSupportWin8::isAvailable()) {
             cfg.setUseWin8PointerInput(false);
@@ -418,22 +499,39 @@ extern "C" int main(int argc, char **argv)
             }
         }
     }
-#endif
+#elif defined QT_HAS_WINTAB_SWITCH
 
-    if (!app.start(args)) {
-        return 1;
+    // Check if WinTab/WinInk has actually activated
+    const bool useWinTabAPI = app.testAttribute(Qt::AA_MSWindowsUseWinTabAPI);
+
+    if (useWinTabAPI != !cfg.useWin8PointerInput()) {
+        cfg.setUseWin8PointerInput(useWinTabAPI);
     }
 
-#if QT_VERSION >= 0x050700
-    app.setAttribute(Qt::AA_CompressHighFrequencyEvents, false);
 #endif
+#endif
+    app.setAttribute(Qt::AA_CompressHighFrequencyEvents, false);
 
     // Set up remote arguments.
     QObject::connect(&app, SIGNAL(messageReceived(QByteArray,QObject*)),
-                    &app, SLOT(remoteArguments(QByteArray,QObject*)));
+                     &app, SLOT(remoteArguments(QByteArray,QObject*)));
 
     QObject::connect(&app, SIGNAL(fileOpenRequest(QString)),
-                    &app, SLOT(fileOpenRequested(QString)));
+                     &app, SLOT(fileOpenRequested(QString)));
+
+    // Hardware information
+    KisUsageLogger::write("\nHardware Information\n");
+    KisUsageLogger::write(QString("  GPU Acceleration: %1").arg(kritarc.value("OpenGLRenderer", "auto").toString()));
+    KisUsageLogger::write(QString("  Memory: %1 Mb").arg(KisImageConfig(true).totalRAM()));
+    KisUsageLogger::write(QString("  Number of Cores: %1").arg(QThread::idealThreadCount()));
+    KisUsageLogger::write(QString("  Swap Location: %1\n").arg(KisImageConfig(true).swapDir()));
+
+    KisConfig(true).logImportantSettings();
+
+    if (!app.start(args)) {
+        KisUsageLogger::log("Could not start Krita Application");
+        return 1;
+    }
 
     int state = app.exec();
 
@@ -442,5 +540,9 @@ extern "C" int main(int argc, char **argv)
         kritarc.setValue("canvasState", "OPENGL_SUCCESS");
     }
 
+    if (logUsage) {
+        KisUsageLogger::close();
+    }
+
     return state;
 }
