@@ -42,8 +42,6 @@
 
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
-#include "kis_change_profile_visitor.h"
-#include "kis_colorspace_convert_visitor.h"
 #include "kis_count_visitor.h"
 #include "kis_filter_strategy.h"
 #include "kis_group_layer.h"
@@ -77,9 +75,12 @@
 #include "processing/kis_crop_processing_visitor.h"
 #include "processing/kis_crop_selections_processing_visitor.h"
 #include "processing/kis_transform_processing_visitor.h"
+#include "processing/kis_convert_color_space_processing_visitor.h"
+#include "processing/kis_assign_profile_processing_visitor.h"
 #include "commands_new/kis_image_resize_command.h"
 #include "commands_new/kis_image_set_resolution_command.h"
 #include "commands_new/kis_activate_selection_mask_command.h"
+#include "kis_do_something_command.h"
 #include "kis_composite_progress_proxy.h"
 #include "kis_layer_composition.h"
 #include "kis_wrapped_rect.h"
@@ -104,6 +105,11 @@
 
 #include "kis_time_range.h"
 
+#include "KisRunnableBasedStrokeStrategy.h"
+#include "KisRunnableStrokeJobData.h"
+#include "KisRunnableStrokeJobUtils.h"
+#include "KisRunnableStrokeJobsInterface.h"
+
 // #define SANITY_CHECKS
 
 #ifdef SANITY_CHECKS
@@ -238,7 +244,14 @@ public:
 
     bool tryCancelCurrentStrokeAsync();
 
-    void notifyProjectionUpdatedInPatches(const QRect &rc);
+    void notifyProjectionUpdatedInPatches(const QRect &rc, QVector<KisRunnableStrokeJobData *> &jobs);
+
+    void convertImageColorSpaceImpl(const KoColorSpace *dstColorSpace,
+                                    bool convertLayers,
+                                    KoColorConversionTransformation::Intent renderingIntent,
+                                    KoColorConversionTransformation::ConversionFlags conversionFlags);
+
+    struct SetImageProjectionColorSpace;
 };
 
 KisImage::KisImage(KisUndoStore *undoStore, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
@@ -305,7 +318,7 @@ KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
     m_d->rootLayer = dynamic_cast<KisGroupLayer*>(newRoot.data());
     setRoot(newRoot);
 
-    if (exactCopy || rhs.m_d->isolatedRootNode) {
+    if (exactCopy || rhs.m_d->isolatedRootNode || rhs.m_d->overlaySelectionMask) {
         QQueue<KisNodeSP> linearizedNodes;
         KisLayerUtils::recursiveApplyNodes(rhs.root(),
             [&linearizedNodes](KisNodeSP node) {
@@ -324,9 +337,18 @@ KisImage::KisImage(const KisImage& rhs, KisUndoStore *undoStore, bool exactCopy)
 
                     m_d->isolatedRootNode = node;
                 }
-            });
+
+                if (rhs.m_d->overlaySelectionMask &&
+                    KisNodeSP(rhs.m_d->overlaySelectionMask) == refNode) {
+
+                    m_d->targetOverlaySelectionMask = dynamic_cast<KisSelectionMask*>(node.data());
+                    m_d->overlaySelectionMask = m_d->targetOverlaySelectionMask;
+                    m_d->rootLayer->notifyChildMaskChanged();
+                }
+        });
     }
 
+    m_d->compositions.clear();
     Q_FOREACH (KisLayerCompositionSP comp, rhs.m_d->compositions) {
         m_d->compositions << toQShared(new KisLayerComposition(*comp, this));
     }
@@ -976,63 +998,222 @@ void KisImage::shear(double angleX, double angleY)
               angleX, angleY, 0);
 }
 
+void KisImage::convertLayerColorSpace(KisNodeSP node,
+                                      const KoColorSpace *dstColorSpace,
+                                      KoColorConversionTransformation::Intent renderingIntent,
+                                      KoColorConversionTransformation::ConversionFlags conversionFlags)
+{
+    if (!node->projectionLeaf()->isLayer()) return;
+
+    const KoColorSpace *srcColorSpace = node->colorSpace();
+
+    if (!dstColorSpace || *srcColorSpace == *dstColorSpace) return;
+
+    KUndo2MagicString actionName =
+        kundo2_i18n("Convert Layer Type"); // restored old name for 4.2 branch
+
+    KisImageSignalVector emitSignals;
+    emitSignals << ModifiedSignal;
+
+    KisProcessingApplicator applicator(this, node,
+                                       KisProcessingApplicator::RECURSIVE |
+                                       KisProcessingApplicator::NO_UI_UPDATES,
+                                       emitSignals, actionName);
+
+    applicator.applyVisitor(
+        new KisConvertColorSpaceProcessingVisitor(
+            srcColorSpace, dstColorSpace,
+            renderingIntent, conversionFlags),
+        KisStrokeJobData::CONCURRENT);
+
+    applicator.end();
+}
+
+struct KisImage::KisImagePrivate::SetImageProjectionColorSpace : public KisCommandUtils::FlipFlopCommand
+{
+    SetImageProjectionColorSpace(const KoColorSpace *cs, KisImageWSP image,
+                                 State initialState, KUndo2Command *parent = 0)
+        : KisCommandUtils::FlipFlopCommand(initialState, parent),
+          m_cs(cs),
+          m_image(image)
+    {
+    }
+
+    void partA() override {
+        KisImageSP image = m_image;
+
+        if (image) {
+            image->setProjectionColorSpace(m_cs);
+        }
+    }
+
+private:
+    const KoColorSpace *m_cs;
+    KisImageWSP m_image;
+};
+
+void KisImage::KisImagePrivate::convertImageColorSpaceImpl(const KoColorSpace *dstColorSpace,
+                                                           bool convertLayers,
+                                                           KoColorConversionTransformation::Intent renderingIntent,
+                                                           KoColorConversionTransformation::ConversionFlags conversionFlags)
+{
+    const KoColorSpace *srcColorSpace = this->colorSpace;
+
+    if (!dstColorSpace || *srcColorSpace == *dstColorSpace) return;
+
+    const KUndo2MagicString actionName =
+        convertLayers ?
+        kundo2_i18n("Convert Image Color Space") :
+        kundo2_i18n("Convert Projection Color Space");
+
+    KisImageSignalVector emitSignals;
+    emitSignals << ColorSpaceChangedSignal;
+    emitSignals << ModifiedSignal;
+
+    KisProcessingApplicator applicator(q, this->rootLayer,
+                                       KisProcessingApplicator::RECURSIVE |
+                                       KisProcessingApplicator::NO_UI_UPDATES,
+                                       emitSignals, actionName);
+
+    applicator.applyCommand(
+        new KisImagePrivate::SetImageProjectionColorSpace(dstColorSpace,
+                                                          KisImageWSP(q),
+                                                          KisCommandUtils::FlipFlopCommand::INITIALIZING),
+        KisStrokeJobData::BARRIER);
+
+    if (convertLayers) {
+        applicator.applyVisitor(
+                    new KisConvertColorSpaceProcessingVisitor(
+                        srcColorSpace, dstColorSpace,
+                        renderingIntent, conversionFlags),
+                    KisStrokeJobData::CONCURRENT);
+    } else {
+        applicator.applyCommand(
+            new KisDoSomethingCommand<
+                    KisDoSomethingCommandOps::ResetOp, KisGroupLayerSP>
+                    (this->rootLayer, false));
+        applicator.applyCommand(
+            new KisDoSomethingCommand<
+                    KisDoSomethingCommandOps::ResetOp, KisGroupLayerSP>
+                    (this->rootLayer, true));
+    }
+
+    applicator.applyCommand(
+        new KisImagePrivate::SetImageProjectionColorSpace(srcColorSpace,
+                                                          KisImageWSP(q),
+                                                          KisCommandUtils::FlipFlopCommand::FINALIZING),
+        KisStrokeJobData::BARRIER);
+
+
+    applicator.end();
+}
+
 void KisImage::convertImageColorSpace(const KoColorSpace *dstColorSpace,
                                       KoColorConversionTransformation::Intent renderingIntent,
                                       KoColorConversionTransformation::ConversionFlags conversionFlags)
 {
-    if (!dstColorSpace) return;
-
-    const KoColorSpace *srcColorSpace = m_d->colorSpace;
-
-    undoAdapter()->beginMacro(kundo2_i18n("Convert Image Color Space"));
-    undoAdapter()->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-    undoAdapter()->addCommand(new KisImageSetProjectionColorSpaceCommand(KisImageWSP(this), dstColorSpace));
-
-    KisColorSpaceConvertVisitor visitor(this, srcColorSpace, dstColorSpace, renderingIntent, conversionFlags);
-    m_d->rootLayer->accept(visitor);
-
-    undoAdapter()->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-    undoAdapter()->endMacro();
-
-    setModified();
+    m_d->convertImageColorSpaceImpl(dstColorSpace, true, renderingIntent, conversionFlags);
 }
 
-bool KisImage::assignImageProfile(const KoColorProfile *profile)
+void KisImage::convertImageProjectionColorSpace(const KoColorSpace *dstColorSpace)
+{
+    m_d->convertImageColorSpaceImpl(dstColorSpace, false,
+                                    KoColorConversionTransformation::internalRenderingIntent(),
+                                    KoColorConversionTransformation::internalConversionFlags());
+}
+
+
+bool KisImage::assignLayerProfile(KisNodeSP node, const KoColorProfile *profile)
+{
+    const KoColorSpace *srcColorSpace = node->colorSpace();
+
+    if (!node->projectionLeaf()->isLayer()) return false;
+    if (!profile || *srcColorSpace->profile() == *profile) return false;
+
+    KUndo2MagicString actionName = kundo2_i18n("Assign Profile to Layer");
+
+    KisImageSignalVector emitSignals;
+    emitSignals << ModifiedSignal;
+
+    const KoColorSpace *dstColorSpace = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
+    if (!dstColorSpace) return false;
+
+    KisProcessingApplicator applicator(this, node,
+                                       KisProcessingApplicator::RECURSIVE |
+                                       KisProcessingApplicator::NO_UI_UPDATES,
+                                       emitSignals, actionName);
+
+    applicator.applyVisitor(
+        new KisAssignProfileProcessingVisitor(
+            srcColorSpace, dstColorSpace),
+        KisStrokeJobData::CONCURRENT);
+
+    applicator.end();
+
+    return true;
+}
+
+
+bool KisImage::assignImageProfile(const KoColorProfile *profile, bool blockAllUpdates)
 {
     if (!profile) return false;
 
-    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
-    const KoColorSpace *srcCs = colorSpace();
+    const KoColorSpace *srcColorSpace = m_d->colorSpace;
+    bool imageProfileIsSame = *srcColorSpace->profile() == *profile;
 
-    if (!dstCs) return false;
+    imageProfileIsSame &=
+        !KisLayerUtils::recursiveFindNode(m_d->rootLayer,
+            [profile] (KisNodeSP node) {
+                return *node->colorSpace()->profile() != *profile;
+            });
 
-    m_d->colorSpace = dstCs;
+    if (imageProfileIsSame) {
+        dbgImage << "Trying to set the same image profile again" << ppVar(srcColorSpace->profile()->name()) << ppVar(profile->name());
+        return true;
+    }
 
-    KisChangeProfileVisitor visitor(srcCs, dstCs);
-    bool retval = m_d->rootLayer->accept(visitor);
-    m_d->signalRouter.emitNotification(ProfileChangedSignal);
-    return retval;
+    KUndo2MagicString actionName = kundo2_i18n("Assign Profile");
 
-}
+    KisImageSignalVector emitSignals;
+    emitSignals << ProfileChangedSignal;
+    emitSignals << ModifiedSignal;
 
-void KisImage::convertProjectionColorSpace(const KoColorSpace *dstColorSpace)
-{
-    if (*m_d->colorSpace == *dstColorSpace) return;
+    const KoColorSpace *dstColorSpace = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
+    if (!dstColorSpace) return false;
 
-    undoAdapter()->beginMacro(kundo2_i18n("Convert Projection Color Space"));
-    undoAdapter()->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-    undoAdapter()->addCommand(new KisImageSetProjectionColorSpaceCommand(KisImageWSP(this), dstColorSpace));
-    undoAdapter()->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-    undoAdapter()->endMacro();
+    KisProcessingApplicator applicator(this, m_d->rootLayer,
+                                       KisProcessingApplicator::RECURSIVE |
+                                       (!blockAllUpdates ?
+                                            KisProcessingApplicator::NO_UI_UPDATES :
+                                            KisProcessingApplicator::NO_IMAGE_UPDATES),
+                                       emitSignals, actionName);
 
-    setModified();
+    applicator.applyCommand(
+        new KisImagePrivate::SetImageProjectionColorSpace(dstColorSpace,
+                                                          KisImageWSP(this),
+                                                          KisCommandUtils::FlipFlopCommand::INITIALIZING),
+        KisStrokeJobData::BARRIER);
+
+    applicator.applyVisitor(
+        new KisAssignProfileProcessingVisitor(
+            srcColorSpace, dstColorSpace),
+        KisStrokeJobData::CONCURRENT);
+
+    applicator.applyCommand(
+        new KisImagePrivate::SetImageProjectionColorSpace(srcColorSpace,
+                                                          KisImageWSP(this),
+                                                          KisCommandUtils::FlipFlopCommand::FINALIZING),
+        KisStrokeJobData::BARRIER);
+
+
+    applicator.end();
+
+    return true;
 }
 
 void KisImage::setProjectionColorSpace(const KoColorSpace * colorSpace)
 {
     m_d->colorSpace = colorSpace;
-    m_d->rootLayer->resetCache();
-    m_d->signalRouter.emitNotification(ColorSpaceChangedSignal);
 }
 
 const KoColorSpace * KisImage::colorSpace() const
@@ -1253,6 +1434,11 @@ KisPostExecutionUndoAdapter* KisImage::postExecutionUndoAdapter() const
         &m_d->postExecutionUndoAdapter;
 }
 
+const KUndo2Command* KisImage::lastExecutedCommand() const
+{
+    return m_d->undoStore->presentCommand();
+}
+
 void KisImage::setUndoStore(KisUndoStore *undoStore)
 {
 
@@ -1399,7 +1585,7 @@ KisStrokeId KisImage::startStroke(KisStrokeStrategy *strokeStrategy)
     return m_d->scheduler.startStroke(strokeStrategy);
 }
 
-void KisImage::KisImagePrivate::notifyProjectionUpdatedInPatches(const QRect &rc)
+void KisImage::KisImagePrivate::notifyProjectionUpdatedInPatches(const QRect &rc, QVector<KisRunnableStrokeJobData*> &jobs)
 {
     KisImageConfig imageConfig(true);
     int patchWidth = imageConfig.updatePatchWidth();
@@ -1410,20 +1596,21 @@ void KisImage::KisImagePrivate::notifyProjectionUpdatedInPatches(const QRect &rc
             QRect patchRect(x, y, patchWidth, patchHeight);
             patchRect &= rc;
 
-            QtConcurrent::run(std::bind(&KisImage::notifyProjectionUpdated, q, patchRect));
+            KritaUtils::addJobConcurrent(jobs, std::bind(&KisImage::notifyProjectionUpdated, q, patchRect));
         }
     }
 }
 
 bool KisImage::startIsolatedMode(KisNodeSP node)
 {
-    struct StartIsolatedModeStroke : public KisSimpleStrokeStrategy {
+    struct StartIsolatedModeStroke : public KisRunnableBasedStrokeStrategy {
         StartIsolatedModeStroke(KisNodeSP node, KisImageSP image)
-            : KisSimpleStrokeStrategy("start-isolated-mode", kundo2_noi18n("start-isolated-mode")),
+            : KisRunnableBasedStrokeStrategy("start-isolated-mode", kundo2_noi18n("start-isolated-mode")),
               m_node(node),
               m_image(image)
         {
             this->enableJob(JOB_INIT, true, KisStrokeJobData::SEQUENTIAL, KisStrokeJobData::EXCLUSIVE);
+            this->enableJob(JOB_DOSTROKE, true);
             setClearsRedoOnStart(false);
         }
 
@@ -1437,7 +1624,10 @@ bool KisImage::startIsolatedMode(KisNodeSP node)
 
             // the GUI uses our thread to do the color space conversion so we
             // need to emit this signal in multiple threads
-            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds());
+            QVector<KisRunnableStrokeJobData*> jobs;
+            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
+            this->runnableJobsInterface()->addRunnableJobs(jobs);
+
 
             m_image->invalidateAllFrames();
         }
@@ -1457,12 +1647,13 @@ void KisImage::stopIsolatedMode()
 {
     if (!m_d->isolatedRootNode)  return;
 
-    struct StopIsolatedModeStroke : public KisSimpleStrokeStrategy {
+    struct StopIsolatedModeStroke : public KisRunnableBasedStrokeStrategy {
         StopIsolatedModeStroke(KisImageSP image)
-            : KisSimpleStrokeStrategy("stop-isolated-mode", kundo2_noi18n("stop-isolated-mode")),
+            : KisRunnableBasedStrokeStrategy("stop-isolated-mode", kundo2_noi18n("stop-isolated-mode")),
               m_image(image)
         {
             this->enableJob(JOB_INIT);
+            this->enableJob(JOB_DOSTROKE, true);
             setClearsRedoOnStart(false);
         }
 
@@ -1474,10 +1665,13 @@ void KisImage::stopIsolatedMode()
 
             emit m_image->sigIsolatedModeChanged();
 
+            m_image->invalidateAllFrames();
+
             // the GUI uses our thread to do the color space conversion so we
             // need to emit this signal in multiple threads
-            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds());
-            m_image->invalidateAllFrames();
+            QVector<KisRunnableStrokeJobData*> jobs;
+            m_image->m_d->notifyProjectionUpdatedInPatches(m_image->bounds(), jobs);
+            this->runnableJobsInterface()->addRunnableJobs(jobs);
 
             // TODO: Substitute notifyProjectionUpdated() with this code
             // when update optimization is implemented
@@ -1603,6 +1797,11 @@ void KisImage::addSpontaneousJob(KisSpontaneousJob *spontaneousJob)
     m_d->scheduler.addSpontaneousJob(spontaneousJob);
 }
 
+bool KisImage::hasUpdatesRunning() const
+{
+    return m_d->scheduler.hasUpdatesRunning();
+}
+
 void KisImage::setProjectionUpdatesFilter(KisProjectionUpdatesFilterSP filter)
 {
     // update filters are *not* recursive!
@@ -1631,6 +1830,21 @@ void KisImage::disableUIUpdates()
     m_d->disableUIUpdateSignals.ref();
 }
 
+void KisImage::notifyBatchUpdateStarted()
+{
+    m_d->signalRouter.emitNotifyBatchUpdateStarted();
+}
+
+void KisImage::notifyBatchUpdateEnded()
+{
+    m_d->signalRouter.emitNotifyBatchUpdateEnded();
+}
+
+void KisImage::notifyUIUpdateCompleted(const QRect &rc)
+{
+    notifyProjectionUpdated(rc);
+}
+
 QVector<QRect> KisImage::enableUIUpdates()
 {
     m_d->disableUIUpdateSignals.deref();
@@ -1868,8 +2082,9 @@ bool KisImage::levelOfDetailBlocked() const
     return m_d->blockLevelOfDetail;
 }
 
-void KisImage::notifyNodeCollpasedChanged()
+void KisImage::nodeCollapsedChanged(KisNode * node)
 {
+    Q_UNUSED(node);
     emit sigNodeCollapsedChanged();
 }
 
