@@ -48,6 +48,7 @@
 #include <KoDialog.h>
 #include <KisImportExportErrorCode.h>
 #include <KoDocumentResourceManager.h>
+#include <KoMD5Generator.h>
 
 #include <KisUsageLogger.h>
 #include <klocalizedstring.h>
@@ -96,6 +97,7 @@
 #include <kis_signal_auto_connection.h>
 #include <kis_canvas_widget_base.h>
 #include "kis_layer_utils.h"
+#include "kis_selection_mask.h"
 
 // Local
 #include "KisViewManager.h"
@@ -392,7 +394,7 @@ public:
 
         if (!m_locked) {
             m_image->requestStrokeEnd();
-            QApplication::processEvents();
+            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 
             // one more try...
             m_locked = std::try_lock(m_imageLock, m_savingLock) < 0;
@@ -561,10 +563,24 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
         QString suffix = cfg.readEntry<QString>("backupfilesuffix", "~");
 
         if (numOfBackupsKept == 1) {
-            KBackup::simpleBackupFile(job.filePath, backupDir, suffix);
+            if (!KBackup::simpleBackupFile(job.filePath, backupDir, suffix)) {
+                qWarning() << "Failed to create simple backup file!" << job.filePath << backupDir << suffix;
+                KisUsageLogger::log(QString("Failed to create a simple backup for %1 in %2.").arg(job.filePath).arg(backupDir.isEmpty() ? "the same location as the file" : backupDir));
+                return false;
+            }
+            else {
+                KisUsageLogger::log(QString("Create a simple backup for %1 in %2.").arg(job.filePath).arg(backupDir.isEmpty() ? "the same location as the file" : backupDir));
+            }
         }
         else if (numOfBackupsKept > 2) {
-            KBackup::numberedBackupFile(job.filePath, backupDir, suffix, numOfBackupsKept);
+            if (!KBackup::numberedBackupFile(job.filePath, backupDir, suffix, numOfBackupsKept)) {
+                qWarning() << "Failed to create numbered backup file!" << job.filePath << backupDir << suffix;
+                KisUsageLogger::log(QString("Failed to create a numbered backup for %2.").arg(job.filePath).arg(backupDir.isEmpty() ? "the same location as the file" : backupDir));
+                return false;
+            }
+            else {
+                KisUsageLogger::log(QString("Create a simple backup for %1 in %2.").arg(job.filePath).arg(backupDir.isEmpty() ? "the same location as the file" : backupDir));
+            }
         }
     }
 
@@ -743,7 +759,7 @@ void KisDocument::setFileBatchMode(const bool batchMode)
 KisDocument* KisDocument::lockAndCloneForSaving()
 {
     // force update of all the asynchronous nodes before cloning
-    QApplication::processEvents();
+    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
     KisLayerUtils::forceAllDelayedNodesUpdate(d->image->root());
 
     KisMainWindow *window = KisPart::instance()->currentMainwindow();
@@ -839,6 +855,11 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
         }
     }
 
+    if (clonedDocument->image()->hasOverlaySelectionMask()) {
+        clonedDocument->image()->setOverlaySelectionMask(0);
+        waitForImage(clonedDocument->image());
+    }
+
     KIS_SAFE_ASSERT_RECOVER(clonedDocument->image()->isIdle()) {
         waitForImage(clonedDocument->image());
     }
@@ -886,28 +907,38 @@ bool KisDocument::initiateSavingInBackground(const QString actionName,
 
 void KisDocument::slotChildCompletedSavingInBackground(KisImportExportErrorCode status, const QString &errorMessage)
 {
-    KIS_SAFE_ASSERT_RECOVER(!d->savingMutex.tryLock()) {
+    KIS_ASSERT_RECOVER_RETURN(isSaving());
+
+    KIS_ASSERT_RECOVER(d->backgroundSaveDocument) {
         d->savingMutex.unlock();
         return;
     }
 
-    KIS_SAFE_ASSERT_RECOVER_RETURN(d->backgroundSaveDocument);
-
     if (d->backgroundSaveJob.flags & KritaUtils::SaveInAutosaveMode) {
         d->backgroundSaveDocument->d->isAutosaving = false;
     }
 
     d->backgroundSaveDocument.take()->deleteLater();
-    d->savingMutex.unlock();
 
-    KIS_SAFE_ASSERT_RECOVER_RETURN(d->backgroundSaveJob.isValid());
+    KIS_ASSERT_RECOVER(d->backgroundSaveJob.isValid()) {
+        d->savingMutex.unlock();
+        return;
+    }
+
     const KritaUtils::ExportFileJob job = d->backgroundSaveJob;
     d->backgroundSaveJob = KritaUtils::ExportFileJob();
 
-    KisUsageLogger::log(QString("Completed saving %1 (mime: %2). Result: %3")
+    // unlock at the very end
+    d->savingMutex.unlock();
+
+
+    QFileInfo fi(job.filePath);
+    KisUsageLogger::log(QString("Completed saving %1 (mime: %2). Result: %3. Size: %4. MD5 Hash: %5")
                         .arg(job.filePath)
                         .arg(QString::fromLatin1(job.mimeType))
-                        .arg(!status.isOk() ? exportErrorToUserMessage(status, errorMessage) : "OK"));
+                        .arg(!status.isOk() ? exportErrorToUserMessage(status, errorMessage) : "OK")
+                        .arg(fi.size())
+                        .arg(QString::fromLatin1(KoMD5Generator().generateHash(job.filePath).toHex())));
 
     emit sigCompleteBackgroundSaving(job, status, errorMessage);
 }
@@ -919,6 +950,8 @@ void KisDocument::slotAutoSaveImpl(std::unique_ptr<KisDocument> &&optionalCloned
 
     emit statusBarMessage(i18n("Autosaving... %1", autoSaveFileName), successMessageTimeout);
 
+    KisUsageLogger::log(QString("Autosaving: %1").arg(autoSaveFileName));
+
     const bool hadClonedDocument = bool(optionalClonedDocument);
     bool started = false;
 
@@ -1197,7 +1230,7 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
         if (QFile::exists(asf)) {
             KisApplication *kisApp = static_cast<KisApplication*>(qApp);
             kisApp->hideSplashScreen();
-            //dbgUI <<"asf=" << asf;
+            //qDebug() <<"asf=" << asf;
             // ## TODO compare timestamps ?
             int res = QMessageBox::warning(0,
                                            i18nc("@title:window", "Krita"),
@@ -1209,6 +1242,7 @@ bool KisDocument::openUrl(const QUrl &_url, OpenFlags flags)
                 autosaveOpened = true;
                 break;
             case QMessageBox::No :
+                KisUsageLogger::log(QString("Removing autosave file: %1").arg(asf));
                 QFile::remove(asf);
                 break;
             default: // Cancel
@@ -1282,7 +1316,7 @@ public:
 bool KisDocument::openFile()
 {
     //dbgUI <<"for" << localFilePath();
-    if (!QFile::exists(localFilePath())) {
+    if (!QFile::exists(localFilePath()) && !fileBatchMode()) {
         QMessageBox::critical(0, i18nc("@title:window", "Krita"), i18n("File %1 does not exist.", localFilePath()));
         return false;
     }
@@ -1325,7 +1359,7 @@ bool KisDocument::openFile()
             updater->cancel();
         }
         QString msg = status.errorMessage();
-        if (!msg.isEmpty()) {
+        if (!msg.isEmpty() && !fileBatchMode()) {
             DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
                                 i18n("Could not open %2.\nReason: %1.", msg, prettyPathOrUrl()),
                                 errorMessage().split("\n") + warningMessage().split("\n"));
@@ -1333,7 +1367,7 @@ bool KisDocument::openFile()
         }
         return false;
     }
-    else if (!warningMessage().isEmpty()) {
+    else if (!warningMessage().isEmpty() && !fileBatchMode()) {
         DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
                             i18n("There were problems opening %1.", prettyPathOrUrl()),
                             warningMessage().split("\n"));
@@ -1509,20 +1543,16 @@ QString KisDocument::warningMessage() const
 
 void KisDocument::removeAutoSaveFiles(const QString &autosaveBaseName, bool wasRecovered)
 {
-    //qDebug() << "removeAutoSaveFiles";
     // Eliminate any auto-save file
     QString asf = generateAutoSaveFileName(autosaveBaseName);   // the one in the current dir
-
-    //qDebug() << "\tfilename:" << asf << "exists:" << QFile::exists(asf);
     if (QFile::exists(asf)) {
-        //qDebug() << "\tremoving autosavefile" << asf;
+        KisUsageLogger::log(QString("Removing autosave file: %1").arg(asf));
         QFile::remove(asf);
     }
     asf = generateAutoSaveFileName(QString());   // and the one in $HOME
 
-    //qDebug() << "Autsavefile in $home" << asf;
     if (QFile::exists(asf)) {
-        //qDebug() << "\tremoving autsavefile 2" << asf;
+        KisUsageLogger::log(QString("Removing autosave file: %1").arg(asf));
         QFile::remove(asf);
     }
 
@@ -1537,6 +1567,7 @@ void KisDocument::removeAutoSaveFiles(const QString &autosaveBaseName, bool wasR
                 rex.match(QFileInfo(autosaveBaseName).fileName()).hasMatch() &&
                 QFile::exists(autosaveBaseName)) {
 
+            KisUsageLogger::log(QString("Removing autosave file: %1").arg(autosaveBaseName));
             QFile::remove(autosaveBaseName);
         }
     }
@@ -1786,6 +1817,8 @@ bool KisDocument::newImage(const QString& name,
     image->setResolution(imageResolution, imageResolution);
 
     image->assignImageProfile(cs->profile());
+    image->waitForDone();
+
     documentInfo()->setAboutInfo("title", name);
     documentInfo()->setAboutInfo("abstract", description);
 
@@ -1835,13 +1868,14 @@ bool KisDocument::newImage(const QString& name,
     cfg.setDefaultColorDepth(image->colorSpace()->colorDepthId().id());
     cfg.defColorProfile(image->colorSpace()->profile()->name());
 
-    KisUsageLogger::log(i18n("Created image \"%1\", %2 * %3 pixels, %4 dpi. Color model: %6 %5 (%7). Layers: %8"
-                             , name
-                             , width, height
-                             , imageResolution * 72.0
-                             , image->colorSpace()->colorModelId().name(), image->colorSpace()->colorDepthId().name()
-                             , image->colorSpace()->profile()->name()
-                             , numberOfLayers));
+    KisUsageLogger::log(QString("Created image \"%1\", %2 * %3 pixels, %4 dpi. Color model: %6 %5 (%7). Layers: %8")
+                             .arg(name)
+                             .arg(width).arg(height)
+                             .arg(imageResolution * 72.0)
+                             .arg(image->colorSpace()->colorModelId().name())
+                             .arg(image->colorSpace()->colorDepthId().name())
+                             .arg(image->colorSpace()->profile()->name())
+                             .arg(numberOfLayers));
 
     QApplication::restoreOverrideCursor();
 
