@@ -409,7 +409,7 @@ void KisViewManager::setCurrentView(KisView *view)
         d->gamutCheck->setChecked(imageView->gamutCheck());
 
         // Wait for the async image to have loaded
-        KisDocument* doc = view->document();
+        KisDocument* doc = imageView->document();
 
         if (KisConfig(true).readEntry<bool>("EnablePositionLabel", false)) {
             connect(d->currentImageView->canvasController()->proxyObject,
