@@ -31,7 +31,7 @@
 #include <kis_config.h>
 
 #include "kis_global.h"
-#include <squeezedcombobox.h>
+#include <KisSqueezedComboBox.h>
 
 #include "ui_wdggeneralsettings.h"
 #include "ui_wdgdisplaysettings.h"
@@ -85,10 +85,12 @@ public:
     int favoritePresets();
     bool showCanvasMessages();
     bool compressKra();
+    bool useZip64();
     bool toolOptionsInDocker();
+    bool kineticScrollingEnabled();
     int kineticScrollingGesture();
     int kineticScrollingSensitivity();
-    bool kineticScrollingScrollbar();
+    bool kineticScrollingHiddenScrollbars();
     bool switchSelectionCtrlAlt();
     bool convertToImageColorspaceOnImport();
 
@@ -172,7 +174,7 @@ public:
     WdgColorSettings  *m_page;
     QButtonGroup m_pasteBehaviourGroup;
     QList<QLabel*> m_monitorProfileLabels;
-    QList<SqueezedComboBox*> m_monitorProfileWidgets;
+    QList<KisSqueezedComboBox*> m_monitorProfileWidgets;
 };
 
 //=======================
@@ -195,6 +197,7 @@ public:
 
 private Q_SLOTS:
     void slotTabletTest();
+    void slotResolutionSettings();
 
 public:
     void setDefault();
@@ -277,6 +280,7 @@ public:
     void setDefault();
 protected Q_SLOTS:
     void slotUseOpenGLToggled(bool isChecked);
+    void slotPreferredSurfaceFormatChanged(int index);
 
 public:
 };
@@ -319,17 +323,14 @@ class KisDlgPreferences : public KPageDialog
 
 public:
 
-    static bool editPreferences();
-
-
-protected:
-
     KisDlgPreferences(QWidget *parent = 0, const char *name = 0);
     ~KisDlgPreferences() override;
 
+    bool editPreferences();
+
     void showEvent(QShowEvent *event) override;
 
-protected:
+private:
 
     GeneralTab *m_general;
     ShortcutSettingsTab  *m_shortcutSettings;
@@ -343,10 +344,14 @@ protected:
 
     QList<KPageWidgetItem*> m_pages;
 
-protected Q_SLOTS:
+private Q_SLOTS:
 
+    void slotButtonClicked(QAbstractButton *button);
     void slotDefault();
 
+private:
+
+    bool m_cancelClicked {false};
 };
 
 #endif
