@@ -165,11 +165,11 @@ void KisOpenGL::initialize()
     debugOut << "\n  supportsOpenGLES:" << bool(g_supportedRenderers & RendererOpenGLES);
     debugOut << "\n  isQtPreferOpenGLES:" << bool(g_rendererPreferredByQt == RendererOpenGLES);
 #endif
-    debugOut << "\n== log ==\n";
-    debugOut.noquote();
-    debugOut << g_surfaceFormatDetectionLog;
-    debugOut.resetFormat();
-    debugOut << "\n== end log ==";
+//    debugOut << "\n== log ==\n";
+//    debugOut.noquote();
+//    debugOut << g_surfaceFormatDetectionLog;
+//    debugOut.resetFormat();
+//    debugOut << "\n== end log ==";
 
     dbgOpenGL.noquote().nospace() << g_debugText;
     KisUsageLogger::write(g_debugText);
