@@ -157,6 +157,10 @@ KisOpenGLCanvas2::KisOpenGLCanvas2(KisCanvas2 *canvas,
                                                      colorConverter->renderingIntent(),
                                                      colorConverter->conversionFlags());
 
+    connect(d->openGLImageTextures.data(),
+            SIGNAL(sigShowFloatingMessage(QString, int, bool)),
+            SLOT(slotShowFloatingMessage(QString, int, bool)));
+
     setAcceptDrops(true);
     setAutoFillBackground(false);
 
@@ -898,6 +902,11 @@ void KisOpenGLCanvas2::slotPixelGridModeChanged()
     update();
 }
 
+void KisOpenGLCanvas2::slotShowFloatingMessage(const QString &message, int timeout, bool priority)
+{
+    canvas()->imageView()->showFloatingMessage(message, QIcon(), timeout, priority ? KisFloatingMessage::High : KisFloatingMessage::Medium);
+}
+
 QVariant KisOpenGLCanvas2::inputMethodQuery(Qt::InputMethodQuery query) const
 {
     return processInputMethodQuery(query);
