@@ -57,7 +57,7 @@
 #include <KoPathShapeMarkerCommand.h>
 #include <KoCanvasBase.h>
 #include <KoCanvasController.h>
-#include <KoCanvasResourceManager.h>
+#include <KoCanvasResourceProvider.h>
 #include <KoDocumentResourceManager.h>
 #include <KoSelection.h>
 #include <KoShapeController.h>
@@ -69,6 +69,7 @@
 #include <KoFillConfigWidget.h>
 #include "kis_canvas_resource_provider.h"
 #include "kis_acyclic_signal_connector.h"
+#include <kis_signal_compressor.h>
 
 // Krita
 #include "kis_double_parse_unit_spin_box.h"
@@ -79,9 +80,9 @@ public:
     CapNJoinMenu(QWidget *parent = 0);
     QSize sizeHint() const override;
 
-    KisDoubleParseUnitSpinBox *miterLimit;
-    QButtonGroup        *capGroup;
-    QButtonGroup        *joinGroup;
+    KisDoubleParseUnitSpinBox *miterLimit {0};
+    QButtonGroup *capGroup {0};
+    QButtonGroup *joinGroup {0};
 };
 
 CapNJoinMenu::CapNJoinMenu(QWidget *parent)
@@ -90,7 +91,7 @@ CapNJoinMenu::CapNJoinMenu(QWidget *parent)
     QGridLayout *mainLayout = new QGridLayout();
     mainLayout->setMargin(2);
 
-     // The cap group
+    // The cap group
     capGroup = new QButtonGroup(this);
     capGroup->setExclusive(true);
 
@@ -165,34 +166,31 @@ class Q_DECL_HIDDEN KoStrokeConfigWidget::Private
 {
 public:
     Private()
-        : canvas(0),
-        active(true),
-        allowLocalUnitManagement(true),
-        fillConfigWidget(0),
-        noSelectionTrackingMode(false)
+        : selectionChangedCompressor(200, KisSignalCompressor::FIRST_ACTIVE)
     {
     }
 
-    KoLineStyleSelector *lineStyle;
-    KisDoubleParseUnitSpinBox *lineWidth;
-    KoMarkerSelector    *startMarkerSelector;
-    KoMarkerSelector    *midMarkerSelector;
-    KoMarkerSelector    *endMarkerSelector;
+    KoLineStyleSelector *lineStyle {0};
+    KisDoubleParseUnitSpinBox *lineWidth {0};
+    KoMarkerSelector *startMarkerSelector {0};
+    KoMarkerSelector *midMarkerSelector {0};
+    KoMarkerSelector *endMarkerSelector {0};
 
-   CapNJoinMenu *capNJoinMenu;
+    CapNJoinMenu *capNJoinMenu {0};
 
-    QWidget *spacer;
+    QWidget*spacer {0};
 
-    KoCanvasBase *canvas;
+    KoCanvasBase *canvas {0};
 
-    bool active;
-    bool allowLocalUnitManagement;
+    bool active {true};
+    bool allowLocalUnitManagement {false};
 
-    KoFillConfigWidget *fillConfigWidget;
-    bool noSelectionTrackingMode;
+    KoFillConfigWidget *fillConfigWidget {0};
+    bool noSelectionTrackingMode {false};
 
     KisAcyclicSignalConnector shapeChangedAcyclicConnector;
     KisAcyclicSignalConnector resourceManagerAcyclicConnector;
+    KisSignalCompressor selectionChangedCompressor;
 
     std::vector<KisAcyclicSignalConnector::Blocker> deactivationLocks;
 
@@ -204,7 +202,7 @@ KoStrokeConfigWidget::KoStrokeConfigWidget(KoCanvasBase *canvas, QWidget * paren
     : QWidget(parent)
     , d(new Private())
 {
-    // confure GUI
+    // configure GUI
     d->ui = new Ui_KoStrokeConfigWidget();
     d->ui->setupUi(this);
 
@@ -212,34 +210,35 @@ KoStrokeConfigWidget::KoStrokeConfigWidget(KoCanvasBase *canvas, QWidget * paren
 
     { // connect the canvas
         d->shapeChangedAcyclicConnector.connectBackwardVoid(
-            canvas->selectedShapesProxy(), SIGNAL(selectionChanged()),
-            this, SLOT(selectionChanged()));
+                    canvas->selectedShapesProxy(), SIGNAL(selectionChanged()),
+                    &d->selectionChangedCompressor, SLOT(start()));
 
         d->shapeChangedAcyclicConnector.connectBackwardVoid(
-            canvas->selectedShapesProxy(), SIGNAL(selectionContentChanged()),
-            this, SLOT(selectionChanged()));
+                    canvas->selectedShapesProxy(), SIGNAL(selectionContentChanged()),
+                    &d->selectionChangedCompressor, SLOT(start()));
+
+        connect(&d->selectionChangedCompressor, SIGNAL(timeout()), this, SLOT(selectionChanged()));
 
         d->resourceManagerAcyclicConnector.connectBackwardResourcePair(
-            canvas->resourceManager(), SIGNAL(canvasResourceChanged(int, const QVariant&)),
-            this, SLOT(canvasResourceChanged(int, const QVariant &)));
+                    canvas->resourceManager(), SIGNAL(canvasResourceChanged(int,QVariant)),
+                    this, SLOT(canvasResourceChanged(int,QVariant)));
 
         d->canvas = canvas;
     }
 
-
     {
 
-       d->fillConfigWidget = new KoFillConfigWidget(canvas, KoFlake::StrokeFill, false, this);
-       d->fillConfigWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
-       d->ui->fillConfigWidgetLayout->addWidget(d->fillConfigWidget);
-       connect(d->fillConfigWidget, SIGNAL(sigFillChanged()), SIGNAL(sigStrokeChanged()));
+        d->fillConfigWidget = new KoFillConfigWidget(canvas, KoFlake::StrokeFill, true, this);
+        d->fillConfigWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
+        d->ui->fillConfigWidgetLayout->addWidget(d->fillConfigWidget);
+        connect(d->fillConfigWidget, SIGNAL(sigFillChanged()), SIGNAL(sigStrokeChanged()));
     }
 
     d->ui->thicknessLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     d->ui->thicknessLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
     // set min/max/step and value in points, then set actual unit
-    d->ui->lineWidth->setMinMaxStep(0.0, 1000.0, 0.5);
+    d->ui->lineWidth->setMinMaxStep(0.5, 1000.0, 0.5); // if someone wants 0, just set to "none" on UI
     d->ui->lineWidth->setDecimals(2);
     d->ui->lineWidth->setUnit(KoUnit(KoUnit::Point));
     d->ui->lineWidth->setToolTip(i18n("Set line width of actual selection"));
@@ -322,7 +321,7 @@ KoStrokeConfigWidget::KoStrokeConfigWidget(KoCanvasBase *canvas, QWidget * paren
         }
     }
 
-    selectionChanged();
+    d->selectionChangedCompressor.start();
 
     d->fillConfigWidget->activate();
     deactivate();
@@ -338,7 +337,7 @@ void KoStrokeConfigWidget::setNoSelectionTrackingMode(bool value)
     d->fillConfigWidget->setNoSelectionTrackingMode(value);
     d->noSelectionTrackingMode = value;
     if (!d->noSelectionTrackingMode) {
-        selectionChanged();
+        d->selectionChangedCompressor.start();
     }
 }
 
@@ -465,7 +464,8 @@ void KoStrokeConfigWidget::activate()
     d->fillConfigWidget->activate();
 
     if (!d->noSelectionTrackingMode) {
-        selectionChanged();
+        // selectionChanged();
+        d->selectionChangedCompressor.start();
     } else {
         loadCurrentStrokeFillFromResourceServer();
     }
@@ -500,8 +500,8 @@ void KoStrokeConfigWidget::setActive(bool active)
 //------------------------
 
 template <typename ModifyFunction>
-    auto applyChangeToStrokes(KoCanvasBase *canvas, ModifyFunction modifyFunction)
-        -> decltype(modifyFunction(KoShapeStrokeSP()), void())
+auto applyChangeToStrokes(KoCanvasBase *canvas, ModifyFunction modifyFunction)
+-> decltype(modifyFunction(KoShapeStrokeSP()), void())
 {
     KoSelection *selection = canvas->selectedShapesProxy()->selection();
 
@@ -519,10 +519,10 @@ template <typename ModifyFunction>
 void KoStrokeConfigWidget::applyDashStyleChanges()
 {
     applyChangeToStrokes(
-        d->canvas,
-        [this] (KoShapeStrokeSP stroke) {
-            stroke->setLineStyle(lineStyle(), lineDashes());
-        });
+                d->canvas,
+                [this] (KoShapeStrokeSP stroke) {
+        stroke->setLineStyle(lineStyle(), lineDashes());
+    });
 
     emit sigStrokeChanged();
 }
@@ -530,10 +530,10 @@ void KoStrokeConfigWidget::applyDashStyleChanges()
 void KoStrokeConfigWidget::applyLineWidthChanges()
 {
     applyChangeToStrokes(
-        d->canvas,
-        [this] (KoShapeStrokeSP stroke) {
-            stroke->setLineWidth(lineWidth());
-        });
+                d->canvas,
+                [this] (KoShapeStrokeSP stroke) {
+        stroke->setLineWidth(lineWidth());
+    });
 
     emit sigStrokeChanged();
 }
@@ -541,13 +541,13 @@ void KoStrokeConfigWidget::applyLineWidthChanges()
 void KoStrokeConfigWidget::applyJoinCapChanges()
 {
     applyChangeToStrokes(
-        d->canvas,
-        [this] (KoShapeStrokeSP stroke) {
+                d->canvas,
+                [this] (KoShapeStrokeSP stroke) {
 
-            stroke->setCapStyle(static_cast<Qt::PenCapStyle>(d->capNJoinMenu->capGroup->checkedId()));
-            stroke->setJoinStyle(static_cast<Qt::PenJoinStyle>(d->capNJoinMenu->joinGroup->checkedId()));
-            stroke->setMiterLimit(miterLimit());
-        });
+        stroke->setCapStyle(static_cast<Qt::PenCapStyle>(d->capNJoinMenu->capGroup->checkedId()));
+        stroke->setJoinStyle(static_cast<Qt::PenJoinStyle>(d->capNJoinMenu->joinGroup->checkedId()));
+        stroke->setMiterLimit(miterLimit());
+    });
 
     emit sigStrokeChanged();
 }
@@ -668,11 +668,11 @@ void KoStrokeConfigWidget::selectionChanged()
 
     // we need to linearize update order, and force the child widget to update
     // before we start doing it
-    d->fillConfigWidget->forceUpdateOnSelectionChanged();
-
 
     QList<KoShape*> shapes = selection->selectedEditableShapes();
 
+    d->fillConfigWidget->forceUpdateOnSelectionChanged(); // calls shapeChanged() logic
+
     KoShape *shape = !shapes.isEmpty() ? shapes.first() : 0;
 
     const KoShapeStrokeSP stroke = shape ? qSharedPointerDynamicCast<KoShapeStroke>(shape->stroke()) : KoShapeStrokeSP();
@@ -738,22 +738,22 @@ void KoStrokeConfigWidget::selectionChanged()
         }
     }
 
-    const bool lineOptionsVisible =  d->fillConfigWidget->selectedFillIndex() != 0;
+    const bool lineOptionsVisible = (d->fillConfigWidget->selectedFillIndex() != 0);
 
     // This switch statement is to help the tab widget "pages" to be closer to the correct size
     // if we don't do this the internal widgets get rendered, then the tab page has to get resized to
     // fill up the space, then the internal widgets have to resize yet again...causing flicker
     switch(d->fillConfigWidget->selectedFillIndex()) {
-        case 0: // no fill
-            this->setMinimumHeight(130);
-            break;
-        case 1: // solid fill
-             this->setMinimumHeight(200);
-             break;
-        case 2: // gradient fill
-            this->setMinimumHeight(350);
-        case 3: // pattern fill
-            break;
+    case 0: // no fill
+        this->setMinimumHeight(130);
+        break;
+    case 1: // solid fill
+        this->setMinimumHeight(200);
+        break;
+    case 2: // gradient fill
+        this->setMinimumHeight(350);
+    case 3: // pattern fill
+        break;
     }
 
 
@@ -778,10 +778,10 @@ void KoStrokeConfigWidget::selectionChanged()
 void KoStrokeConfigWidget::canvasResourceChanged(int key, const QVariant &value)
 {
     switch (key) {
-    case KoCanvasResourceManager::Unit:
+    case KoCanvasResourceProvider::Unit:
         // we request the whole selection to reload because the
         // unit of the stroke width depends on the selected shape
-        selectionChanged();
+        d->selectionChangedCompressor.start();
         break;
     case KisCanvasResourceProvider::Size:
         if (d->noSelectionTrackingMode) {
