@@ -59,10 +59,10 @@ void HistoryDock::setCanvas(KoCanvasBase *canvas)
             && myCanvas->shapeController()
             && myCanvas->shapeController()->resourceManager()
             && myCanvas->shapeController()->resourceManager()->undoStack()) {
-        KUndo2Stack* undoStack = canvas->shapeController()->resourceManager()->undoStack();
+        KUndo2Stack* undoStack = myCanvas->shapeController()->resourceManager()->undoStack();
 
         m_undoView->setStack(undoStack);
-        KisConfig cfg;
+        KisConfig cfg(true);
         m_undoView->stack()->setUseCumulativeUndoRedo(cfg.useCumulativeUndoRedo());
         m_undoView->stack()->setTimeT1(cfg.stackT1());
         m_undoView->stack()->setTimeT2(cfg.stackT2());
