@@ -26,6 +26,9 @@ class KisJpegTest : public QObject
     Q_OBJECT
 private Q_SLOTS:
     void testFiles();
+    void testImportFromWriteonly();
+    void testExportToReadonly();
+    void testImportIncorrectFormat();
 };
 
 #endif
