@@ -101,6 +101,12 @@ KisBrushOp::KisBrushOp(const KisPaintOpSettingsSP settings, KisPainter *painter,
 
     m_rotationOption.applyFanCornersInfo(this);
 
+    m_precisionOption.setHasImprecisePositionOptions(
+        m_precisionOption.hasImprecisePositionOptions() |
+        m_scatterOption.isChecked() |
+        m_rotationOption.isChecked() |
+        m_airbrushOption.enabled);
+
     KisBrushSP baseBrush = m_brush;
     auto resourcesFactory =
         [baseBrush, settings, painter] () {
